package com.xoado.domain.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.xoado.domain.entity.Skill;
import com.xoado.domain.entity.SkillList;
import com.xoado.domain.exception.BusinessException;
import com.xoado.domain.util.FakeLog;
import com.xoado.domain.util.WordUtil;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.Resource;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

@Service
public class IdentifyService {

    private static SkillList skillList = null;

    @Value("classpath:skill.json")
    private Resource relation;

    private static final String SUFFIX_REGEX = "(吧|啊|嗯|哦|吗)";
    private static final String PREFIX_REGEX = "(请)?((帮|给)我)|(我(想|要|打算|准备))";

    /**
     * 初始化技能对象列表
     * @throws IOException
     */
    private void initSkillList() throws IOException {
        FakeLog.log("init skill list...");
        String relationStr = null;
        InputStream inputStream = relation.getInputStream();
        byte[] bytes = new byte[0];
        bytes = new byte[inputStream.available()];
        inputStream.read(bytes);
        String str = new String(bytes);
        FakeLog.log(str);
        skillList = new SkillList();
        skillList.setSkillList(JSON.parseArray(str,Skill.class));
        FakeLog.log(skillList.getSkillList().toString());
    }

    /**
     * 通过关键字匹配
     * @param context
     * @return
     */
    public List matchSkillByKey(String context){
        FakeLog.log("match skills by keyword...");
        List<Skill> matchedSKills = new ArrayList();
        for(Skill skill:skillList.getSkillList()){
            List<String> keylist = skill.getKeyList();
            for(String key:keylist){
                if(context.contains(key)){
                    matchedSKills.add(skill);
                    break;
                }
            }
        }
        return matchedSKills;
    }

    /**
     * 通过正则匹配
     * @param context
     * @return
     */
    public List<Skill> matchSkillByRegex(String context,List<Skill> matchedSKill){
        FakeLog.log("match skill by regex...");
        List<Skill> matchedSKills = new ArrayList();
        for(Skill skill:matchedSKill){
            List<String> Regexlist = skill.getRegexeList();
            for(String regex:Regexlist){
                if(context.matches(regex)) {
                    matchedSKills.add(skill);
                    break;
                }
            }
        }
        return matchedSKills;
    }

    public JSONObject identifyText(String text) {
        FakeLog.log("format text:"+text);
        String formattext = formatText(text);
        FakeLog.log("start identify text...");
        //初始化技能列表
        if(skillList == null){
            try {
                initSkillList();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        //根据关键字进行初步筛选
        List<Skill> matchedSkills = matchSkillByKey(formattext);
        //根据正则匹配
        if(matchedSkills.size()==0){
            matchedSkills = matchSkillByRegex(formattext,matchedSkills);
        }
//        Skill matchedSkill = null;
//        List wordList = WordUtil.getNlpSeg(text);
//        //TODO 匹配到一个的话直接确定意图，否则用模型进行二次识别。识别模型目前必定识别出结果，无概率计算。
//        if((matchedSkills.size()==1)){
//            matchedSkill = matchedSkills.get(0);
//            FakeLog.log("match only one skill:"+matchedSkill.getName() );
//        }else if(matchedSkills.size()>1){
//            String identifyBySVMResult = SVMService.predict(wordList);
//            matchedSkill = StringUtils.isEmpty(identifyBySVMResult)?null:skillList.getSkillByName(identifyBySVMResult);
//        }
        //直接进行模型测试
        Skill matchedSkill = null;
        List wordList = WordUtil.getNlpSeg(text);
        String identifyBySVMResult = SVMService.predict(wordList);
        matchedSkill = StringUtils.isEmpty(identifyBySVMResult)?null:skillList.getSkillByName(identifyBySVMResult);

        if(matchedSkill == null){
            FakeLog.log("无法识别:"+text);
            return assembleResult("400",text,"无法识别",null);
        }
        return assembleResult("200",text,matchedSkill.getName(),wordList.toString());
    }

    public String formatText(String text) {
        text = text.replaceAll(PREFIX_REGEX,"");
        text = text.replaceAll(SUFFIX_REGEX,"");
        return text;
    }

    private JSONObject assembleResult(String code, String text, String predictResult, String splitWord){
        JSONObject rejson = new JSONObject();
        rejson.put("code",code);
        rejson.put("text",text);
        if(!StringUtils.isEmpty(predictResult)){
            rejson.put("splitWord",splitWord);
        }
        rejson.put("predictResult",predictResult);
        return rejson;
    }
}
