package com.xoado.domain.service;

import com.xoado.domain.svm.comMain;
import com.xoado.domain.util.FakeLog;
import com.xoado.domain.util.WordUtil;
import libsvm.svm;
import libsvm.svm_model;
import libsvm.svm_node;
import libsvm.svm_parameter;
import org.shirdrn.document.processor.TrainDocumentProcessorDriver;

import java.io.*;
import java.util.*;

public class SVMService {

    private static Map<Integer,String> labelNumberMap;

    private static Map<String,String> wordAndNumberMap;

    private static boolean modelIsUseful = true;

    private static svm_model model;


    static{
        //初始化数字序号和对应意图、单词的关系
        initNumberAndLabelOrWordMap();

        initModel();
    }

    private static void initModel(){
        String model_file = "model//model.txt";
        try {
            model = svm.svm_load_model(model_file);
            if (model == null) {
                System.err.print("can't open model file " + model_file + "\n");
                System.exit(1);
            }
        }catch (IOException e) {
            e.printStackTrace();
        }
    }

    //刷新模型和相关数据
    public static void refreshModel() {
        modelIsUseful = false;
        deleteFile("model\\train.txt","model\\model.txt","model\\TestResult.txt","model\\train.txt","model\\terms.txt","model\\labels.txt");
        TrainDocumentProcessorDriver.main(null);
        //歇一会儿让模型生成完毕……
        try{
            comMain.main(null);
        } catch (IOException e) {
            e.printStackTrace();
        }
        initNumberAndLabelOrWordMap();
        initModel();
        modelIsUseful = true;
    }

    private static void initNumberAndLabelOrWordMap(){
        //初始化数字序号和对应意图、单词的关系
        labelNumberMap = new HashMap<>();
        try {
            BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(new File("model\\labels.txt")),
                    "UTF-8"));
            String lineTxt = null;
            while ((lineTxt = br.readLine()) != null) {
                String[] numberAndLabelName = lineTxt.split("\\s+");
                labelNumberMap.put(Integer.parseInt(numberAndLabelName[0]),numberAndLabelName[1]);
            }
            br.close();
            FakeLog.log(labelNumberMap.toString());
        } catch (Exception e) {
            System.err.println("read errors :" + e);
        }
        wordAndNumberMap = new HashMap<>();
        try {
            BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(new File("model\\terms.txt")),
                    "UTF-8"));
            String lineTxt = null;
            while ((lineTxt = br.readLine()) != null) {
                String[] numberAndLabelName = lineTxt.split("\\s+");
                wordAndNumberMap.put(numberAndLabelName[0],numberAndLabelName[1]);
            }
            FakeLog.log(wordAndNumberMap.toString());
            br.close();
        } catch (Exception e) {
            System.err.println("read errors :" + e);
        }
    }

    //删除文件
    private static void deleteFile(String...fileNames){
        for(String fileName:fileNames){
            File file = new File(fileName);
            if(file.exists()){
                file.delete();
            }
        }
    }

    /**
     * 根据分词的结果来预测
     * @param wordList
     * @return
     */
    public static String predict(List wordList) {
        if(!modelIsUseful){
            return "";
        }
        FakeLog.log(wordList.toString());
        //FakeLog.log(labelNumberMap);
        //FakeLog.log(wordAndNumberMap);
        StringBuffer sb = new StringBuffer();
        //label的值
        sb.append("");
        int countExistWord = 0;
        for(Object word :wordList){
            String[] words = String.valueOf(word).split("/");
            if(words.length == 2) {
                String wordWithOutlexicalCategory = words[0];
                String lexicalCategory = words[1];
                if(wordAndNumberMap.get(wordWithOutlexicalCategory)!=null) {
                    sb.append(wordAndNumberMap.get(wordWithOutlexicalCategory)).append(":").append("1").append(" ");
                    countExistWord++;
                }
            }
        }
        //完全不认识的内容直接抛出
        if(countExistWord==0){
            return null;
        }

        //predict_probability为1时支持准确率识别
        int i, predict_probability=1;
        if(predict_probability == 1)
        {
            if(svm.svm_check_probability_model(model)==0)
            {
                System.err.print("Model does not support probabiliy estimates\n");
                System.exit(1);
            }
        }
        else {
            if(svm.svm_check_probability_model(model)!=0)
            {
                FakeLog.log("Model supports probability estimates, but disabled in prediction.\n");
            }
        }
        FakeLog.log("转码后数据：" +sb.toString());
        Map.Entry<Integer,Double> predictResult = null;
        try {
            predictResult = predict(sb.toString(), model, predict_probability);
        } catch (IOException e) {
            e.printStackTrace();
        }
        //TODO 匹配度默认值暂时定为0.5
        String result = null;
        //不考虑匹配度的话直接返回匹配到的意图
        if(predictResult.getValue()==null){
            result = labelNumberMap.get(predictResult.getKey());
        }else{
            if(predictResult.getValue()>0.5){
                result = labelNumberMap.get(predictResult.getKey());
            }
        }
        //完全匹配不到的话，返回null
        return result;
    }

    public static void main(String[] args) throws IOException, InterruptedException {

        //更新训练模型
        boolean rebuildModel = false;
        if(rebuildModel){
            modelIsUseful = false;
            deleteFile("model\\train.txt","model\\model.txt","model\\TestResult.txt","model\\train.txt","model\\terms.txt","model\\labels.txt");
            TrainDocumentProcessorDriver.main(null);
            //歇一会儿让模型生成完毕……
            try{
                comMain.main(null);
            } catch (IOException e) {
                e.printStackTrace();
            }
            initNumberAndLabelOrWordMap();
            modelIsUseful = true;
        }

        String testText = "我想打车去吃外卖";
        //String testText = "我要坐飞机去北京";
        //String testText = "帮我把空调打开";
        //String testText = "给我叫个外卖";
        FakeLog.log( "待预测语料："+testText);
        List wordList = WordUtil.getNlpSeg(testText);
        FakeLog.log(wordList.toString());
        //FakeLog.log(labelNumberMap);
        //FakeLog.log(wordAndNumberMap);
        StringBuffer sb = new StringBuffer();
        sb.append("-1 ");
        for(Object word :wordList){
            String[] words = String.valueOf(word).split("/");
            if(words.length == 2) {
                String wordWithOutlexicalCategory = words[0];
                String lexicalCategory = words[1];
                if(wordAndNumberMap.get(wordWithOutlexicalCategory)!=null) {
                    sb.append(wordAndNumberMap.get(wordWithOutlexicalCategory)).append(":").append("1").append(" ");
                }
            }
        }

        String model_file = "model//model.txt";
        int i, predict_probability=1;

        try
        {
            svm_model model = svm.svm_load_model(model_file);
            if (model == null)
            {
                System.err.print("can't open model file "+model_file+"\n");
                System.exit(1);
            }
            if(predict_probability == 1)
            {
                if(svm.svm_check_probability_model(model)==0)
                {
                    System.err.print("Model does not support probabiliy estimates\n");
                    System.exit(1);
                }
            }
            else
            {
                if(svm.svm_check_probability_model(model)!=0)
                {
                    FakeLog.log("Model supports probability estimates, but disabled in prediction.\n");
                }
            }
            FakeLog.log("转码后数据：" +sb.toString());
            predict(sb.toString(),model,predict_probability);
        }
        catch(FileNotFoundException e)
        {
            e.printStackTrace();
        }
        catch(ArrayIndexOutOfBoundsException e)
        {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private static double atof(String s)
    {
        return Double.valueOf(s).doubleValue();
    }

    private static int atoi(String s)
    {
        return Integer.parseInt(s);
    }

    private static Map.Entry<Integer,Double> predict(String context, svm_model model, int predict_probability) throws IOException
    {
        String predictResult = "";
        int correct = 0;
        int total = 0;
        double error = 0;
        double sump = 0, sumt = 0, sumpp = 0, sumtt = 0, sumpt = 0;
        int svm_type= svm.svm_get_svm_type(model);
        int nr_class=svm.svm_get_nr_class(model);
        double[] prob_estimates=null;

        if(predict_probability == 1)
        {
            if(svm_type == svm_parameter.EPSILON_SVR ||
                    svm_type == svm_parameter.NU_SVR)
            {
                FakeLog.log("Prob. model for test data: target value = predicted value + z,\nz: Laplace distribution e^(-|z|/sigma)/(2sigma),sigma="+svm.svm_get_svr_probability(model)+"\n");
            }
            else
            {
                int[] labels=new int[nr_class];
                svm.svm_get_labels(model,labels);
                prob_estimates = new double[nr_class];
//                output.writeBytes("labels");
//                for(int j=0;j<nr_class;j++)
//                    output.writeBytes(" "+labels[j]);
//                output.writeBytes("\n");
//                for(int j=0;j<nr_class;j++){
//                    FakeLog.log("labels[j]:"+labels[j]);
//                }
            }
        }

            String line = context;

            StringTokenizer st = new StringTokenizer(line," \t\n\r\f:");

            double target_label = atof(st.nextToken());
            int m = st.countTokens()/2;
            svm_node[] x = new svm_node[m];
            for(int j=0;j<m;j++)
            {
                x[j] = new svm_node();
                x[j].index = atoi(st.nextToken());
                x[j].value = atof(st.nextToken());
            }

            Map.Entry<Integer,Double> labelNumberAndEstimate = null;

            double predict_label;
            if (predict_probability==1 && (svm_type==svm_parameter.C_SVC || svm_type==svm_parameter.NU_SVC))
            {
                predict_label = svm.svm_predict_probability(model,x,prob_estimates);
                int key = (int) predict_label;
                FakeLog.log("predict_label:"+ labelNumberMap.get(key));
                labelNumberAndEstimate = new AbstractMap.SimpleEntry<>(key,prob_estimates[key-1]);
                predictResult = labelNumberMap.get(key);
                FakeLog.log("Detailed suitability:");
                for(int j=0;j<nr_class;j++) {
                    FakeLog.log(labelNumberMap.get(j+1) + ":"+prob_estimates[j]);
                }
            }
            else
            {
                predict_label = svm.svm_predict(model,x);
                int key = (int) predict_label;
                FakeLog.log("predict_label:"+ labelNumberMap.get(key));
                predictResult = labelNumberMap.get(key);
                labelNumberAndEstimate = new AbstractMap.SimpleEntry<>(key,null);
            }

//            if(predict_label == target_label)
//                ++correct;
//            error += (predict_label-target_label)*(predict_label-target_label);
//            sump += predict_label;
//            sumt += target_label;
//            sumpp += predict_label*predict_label;
//            sumtt += target_label*target_label;
//            sumpt += predict_label*target_label;

            //这打的是啥日志啊
//        if(svm_type == svm_parameter.EPSILON_SVR ||
//                svm_type == svm_parameter.NU_SVR)
//        {
//            FakeLog.log("Mean squared error = "+error/total+" (regression)\n");
//            FakeLog.log("Squared correlation coefficient = "+
//                    ((total*sumpt-sump*sumt)*(total*sumpt-sump*sumt))/
//                            ((total*sumpp-sump*sump)*(total*sumtt-sumt*sumt))+
//                    " (regression)\n");
//        }
//        else {
//            FakeLog.log("Accuracy = " + (double) correct / total * 100 +
//                    "% (" + correct + "/" + total + ") (classification)\n");
//        }
        return labelNumberAndEstimate;
    }

}
