package org.shirdrn.document.processor.component;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.shirdrn.document.processor.common.AbstractComponent;
import org.shirdrn.document.processor.common.Context;

import java.io.*;

public class BasicInformationCollector extends AbstractComponent {
	
	private static final Log LOG = LogFactory.getLog(BasicInformationCollector.class);
	
	public BasicInformationCollector(Context context) {
		super(context);
	}

	@Override
	public void fire() {
		// get total document count for computing TF-IDF
		int totalDocCount = 0;
		System.out.println(context);
		System.out.println(context.getFDMetadata());
		System.out.println(context.getFDMetadata().getInputRootDir().getAbsolutePath());

		for(String label : context.getFDMetadata().getInputRootDir().list()) {
			context.getVectorMetadata().addLabel(label);
			LOG.info("Add label: label=" + label);
			File labelDir = new File(context.getFDMetadata().getInputRootDir(), label);
			File[] files = labelDir.listFiles(new FileFilter() {
				@Override
				public boolean accept(File pathname) {
					return pathname.getAbsolutePath().endsWith(context.getFDMetadata().getFileExtensionName());
				}
			});
			int totalDocLabelCount = 0;
			for(File file : files) {
				LineNumberReader reader = null;
				FileReader in = null;
				try {
					in = new FileReader(file);
					reader = new LineNumberReader(in);
					reader.skip(Long.MAX_VALUE);
					totalDocLabelCount += reader.getLineNumber();
					reader.close();
				} catch (IOException e) {
					e.printStackTrace();
				} finally {
					try{
						reader.close();
						in.close();
					} catch (IOException e) {
						LOG.warn(e);
					}
				}

			}
			totalDocCount+=totalDocLabelCount;
			context.getVectorMetadata().putLabelledTotalDocCount(label, totalDocLabelCount);
			LOG.info("Put document count: label= " + label + ", docCount=" + totalDocLabelCount);
		}
		LOG.info("Total documents: totalCount= " + totalDocCount);
		context.getVectorMetadata().setTotalDocCount(totalDocCount);
	}

}
